

#ifndef EC_DAEMON_H
#define EC_DAEMON_H

#include <ec.h>
#include <ec_ui.h>
#include <ec_threads.h>
#include <ec_scan.h>
#include <ec_mitm.h>
#include <ec_plugins.h>

#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>

/* proto */
extern void set_daemon_interface(void);


#endif

/* EOF */

// vim:ts=3:expandtab

